import { Host, h } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
import { createFocusTrap } from "focus-trap";
/**
 * @slot openButton - This slot will expand the navigation when clicked.
 * @slot closeButton - This slot will close the navigation when clicked.
 * @slot skipContent - This slot is a keyboard-focus link that lets a screen-reader or keyboard user skip the navigation block and scroll to the end.
 * @slot content - Default slot
 */
export class ContentNavigationBlock {
    constructor() {
        this.hamburger = 'fixed|none';
        this.classes = undefined;
        this.classString = undefined;
        this.ariaExpanded = 'false';
        this.bodyToggle = false;
        this.skipAccessibilityMessage = undefined;
        this.hasOpenButton = false;
        this.hasCloseButton = false;
        this.openButton = undefined;
        this.closeButton = undefined;
        this.contentSlot = undefined;
    }
    componentWillLoad() {
        this.hasOpenButton = !!this.hostElement.querySelector('[slot="openButton"]');
        this.hasCloseButton = !!this.hostElement.querySelector('[slot="closeButton"]');
        WindowResizeService.registerProp(this.hamburger).subscribe(newValue => {
            this.hamburger = newValue;
            this.bodyToggle = this.hamburger === 'fixed';
            document.body.classList.remove('content-navigation-block-open');
        });
        WindowResizeService.registerProp(this.classes).subscribe(newValue => {
            this.classString = newValue;
        });
    }
    componentDidLoad() {
        this.openButton.onclick = () => {
            this.openMenu();
        };
        this.closeButton.onclick = () => {
            this.closeMenu();
        };
    }
    openMenu() {
        this.ariaExpanded = this.ariaExpanded === 'true' ? 'false' : 'true';
        if (this.ariaExpanded === 'false') {
            document.body.classList.remove('content-navigation-block-open');
        }
        else if (this.bodyToggle) {
            document.body.classList.add('content-navigation-block-open');
        }
        this.focusTrap = createFocusTrap(this.contentSlot.parentElement, {
            initialFocus: this.contentSlot,
            fallbackFocus: this.hostElement,
            clickOutsideDeactivates: true,
            onDeactivate: () => {
                this.closeMenu();
                this.focusTrap = null;
                this.ariaExpanded = 'false';
                document.body.classList.remove('content-navigation-block-open');
                setTimeout(() => this.openButton.focus(), 1);
            },
        });
        this.focusTrap.activate();
    }
    closeMenu() {
        if (this.focusTrap) {
            try {
                this.focusTrap.deactivate();
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    render() {
        return (h(Host, { key: '60e3048ab56cc17e7522ce3bccc527c4bed19d84', class: this.classString, "aria-expanded": this.ariaExpanded }, h("div", { key: '17d67b2b8ac3dcb0295f21aaaa90000781904faa', class: "content-navigation-block__skip-content" }, h("slot", { key: 'd2b48b2a5303addf5ba96446b0d62ab58df63bd9', name: "skipContent" })), h("div", { key: 'f761d179d17aba23f753e06b0053c51b5e211d0a', ref: el => (this.openButton = el), class: "content-navigation__open" }, !this.hasOpenButton && (h("button", { key: '1e1ccdc95f16b1a67993dd027cec01bc5acdb4a6' }, h("svg", { key: '81160138c5ba378e1636fa7cad9eca021d3adfc1', class: "default-icon", xmlns: "http://www.w3.org/2000/svg", fill: "none", viewBox: "0 0 24 24", "stroke-width": "1.5", stroke: "currentColor" }, h("path", { key: '839d1da86a66cf9cc5d859ac18e5fcdf07378cd8', "stroke-linecap": "round", "stroke-linejoin": "round", d: "M3.75 6.75h16.5M3.75 12h16.5m-16.5 5.25h16.5" })))), h("slot", { key: 'c5f1cbae43972510679119a34a020ae9f3fcfebb', name: "openButton" })), h("div", { key: '7a46c5d6e94682c3c766f5e3d855f7cbb5bd1a30', class: "content-navigation__content-wrapper" }, h("div", { key: 'b97fff1a6b0865771580cf04adc248af19ca62de', ref: el => (this.closeButton = el), class: "content-navigation__close" }, !this.hasCloseButton && (h("button", { key: '79b6601fe41e9b04d14b79f8a3861a5707ea12a9' }, h("svg", { key: 'd896c72d9aed7383968dd17da37ee1348a453805', class: "default-icon", xmlns: "http://www.w3.org/2000/svg", fill: "none", viewBox: "0 0 24 24", "stroke-width": "1.5", stroke: "currentColor" }, h("path", { key: '12322b22ce35445878f82c3f2d90a8c9de6ed835', "stroke-linecap": "round", "stroke-linejoin": "round", d: "M6 18 18 6M6 6l12 12" })))), h("slot", { key: 'd90e629a54928181707f53286bfc660113ac603a', name: "closeButton" })), h("div", { key: 'f2ba632063737c2ef5e0ddfb4db81e223036a2fc', class: "content-navigation__content", ref: el => (this.contentSlot = el) }, h("slot", { key: 'f3bf538a642cb87013b66b757fe545e32ee17c2e', name: "content" }), h("slot", { key: 'c4d0af20cc31af068d827b5c07bccf625cdf52e0' })))));
    }
    static get is() { return "content-navigation-block"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-navigation-block.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-navigation-block.css"]
        };
    }
    static get properties() {
        return {
            "hamburger": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }, {
                            "name": "options",
                            "text": "fixed"
                        }, {
                            "name": "options",
                            "text": "absolute"
                        }, {
                            "name": "options",
                            "text": "sticky"
                        }, {
                            "name": "options",
                            "text": "static"
                        }, {
                            "name": "options",
                            "text": "none"
                        }],
                    "text": ""
                },
                "attribute": "hamburger",
                "reflect": true,
                "defaultValue": "'fixed|none'"
            },
            "classes": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": true
            }
        };
    }
    static get states() {
        return {
            "classString": {},
            "ariaExpanded": {},
            "bodyToggle": {},
            "skipAccessibilityMessage": {},
            "hasOpenButton": {},
            "hasCloseButton": {},
            "openButton": {},
            "closeButton": {},
            "contentSlot": {}
        };
    }
    static get elementRef() { return "hostElement"; }
}
//# sourceMappingURL=content-navigation-block.js.map
