import { Host, h } from "@stencil/core";
import { WindowResizeService } from "../../services/window-resize-service";
import * as focusTrap from "focus-trap";
import "./libraries/aria-utilities";
import "./libraries/aria-dialog";
/**
 * @slot closeButton - Contents of the overlay close button.
 * @slot header - This is adjacent to the close button, usually a header or title for the overlay.
 * @slot content - Contents that sit inside the overlay.
 */
export class ContentOverlay {
    constructor() {
        this.target = undefined;
        this.open = 'false';
        this.fullScreen = 'true';
        this.classes = undefined;
        this.horizontal = 'center';
        this.vertical = 'center';
        this.closeTitle = 'Close Overlay';
        this.position = 'fixed';
        this.isActive = false;
        this.activeClass = '';
        this.classString = '';
        this.contentSlot = undefined;
        this.isFullScreen = false;
        this.hasCloseButton = false;
        this.originalMarkup = undefined;
    }
    onContentBlockToggle(event) {
        if (this.hostElement.getAttribute('id') === event.detail.getAttribute('id')) {
            event.preventDefault();
            if (!this.isActive) {
                this.openOverlay();
            }
            else {
                this.closeOverlay();
            }
        }
    }
    onContentBlockActivated(event) {
        if (this.hostElement.getAttribute('id') === event.detail.getAttribute('id')) {
            event.preventDefault();
            this.openOverlay();
        }
    }
    onContentBlockDeactivated(event) {
        if (this.hostElement.getAttribute('id') === event.detail.getAttribute('id')) {
            event.preventDefault();
            this.closeOverlay();
        }
    }
    async openOverlay() {
        this.isActive = true;
        document.body.classList.add(`content-overlay-open`, `content-overlay-open--${this.position}`);
        document.body.classList.remove('content-overlay-close');
        if (!this.originalMarkup) {
            this.originalMarkup = this.contentSlot.innerHTML;
        }
        if (this.target) {
            let targetElement = this.target === '#' ? this.hostElement.parentElement : document.querySelector(this.target);
            if (targetElement) {
                targetElement.classList.add('content-block-overlay-container');
                targetElement.appendChild(this.hostElement);
            }
            // } else if (this.isFullScreen) {
            // if (document.querySelectorAll(`#${this.hostElement.getAttribute('id')}`).length === 1) {
            //   document.body.appendChild(this.hostElement);
            // }
        }
        if (this.originalMarkup) {
            this.contentSlot.innerHTML = this.originalMarkup;
            if (this.contentSlot.children[0] === this.contentSlot.querySelector('script[slot="content"][type="text/template"]')) {
                this.contentSlot.innerHTML = this.contentSlot.children[0].innerHTML;
            }
            this.setComputedActiveClass();
            this.focusTrap = focusTrap.createFocusTrap(this.hostElement, {
                fallbackFocus: this.hostElement,
                allowOutsideClick: true,
                clickOutsideDeactivates: true,
                onDeactivate: () => {
                    this.isActive = false;
                    document.body.classList.add('content-overlay-close');
                    document.body.classList.remove(`content-overlay-open`, `content-overlay-open--${this.position}`);
                    this.activeClass = '';
                    this.setComputedActiveClass();
                    this.contentSlot.innerHTML = '';
                },
            });
        }
        this.contentOverlayActivated.emit(this.hostElement);
    }
    async closeOverlay() {
        if (this.focusTrap) {
            try {
                this.focusTrap.deactivate();
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    setComputedActiveClass() {
        this.activeClass = `content-overlay ${this.isActive ? 'content-overlay-active' : 'content-overlay-inactive'}  content-overlay--${this.position}`;
    }
    componentWillLoad() {
        WindowResizeService.registerProp(this.classes).subscribe(newValue => (this.classString = `${newValue}`));
        WindowResizeService.registerProp(this.position).subscribe(newValue => (this.position = newValue));
        WindowResizeService.registerProp(this.horizontal).subscribe(newValue => this.hostElement.setAttribute('horizontal', newValue));
        WindowResizeService.registerProp(this.vertical).subscribe(newValue => this.hostElement.setAttribute('vertical', newValue));
        WindowResizeService.registerProp(this.fullScreen).subscribe(newValue => {
            this.fullScreen = newValue;
            this.isFullScreen = newValue === 'true';
        });
        WindowResizeService.registerProp(this.open).subscribe(newValue => {
            this.hostElement.setAttribute('open', newValue);
        });
        this.hasCloseButton = !!this.hostElement.querySelector('[slot="closeButton"]');
    }
    closeOverlayButtonClicked(event) {
        event.preventDefault();
        this.closeOverlay();
    }
    render() {
        return (h(Host, { key: '70525d441fef22dfd54c93047776b293411de805', class: `${this.activeClass} ${this.classString}`, position: this.position }, h("div", { key: 'f69a2d0a9f62b9a8c6a73106b419023b70528152', class: "content-block-overlay-container" }, h("div", { key: '3eda44dcc57d4704258751e4287baf1223e4b393', class: `content-overlay-background ${this.isActive ? 'active' : ''}` }, h("div", { key: '5ceaa528e484ee2de56093dae89f9a6138a06f50', ref: element => (this.dialogElement = element), role: "dialog", "aria-modal": "true", class: this.isActive ? 'active' : 'hidden', "aria-expanded": this.isActive }, h("div", { key: '7af296f91887e14978090474e1e60e0807bc2684', class: "content-overlay__header" }, h("button", { key: 'e312a9a53f4d8400d8f99b3b85a252a4713d9c24', onClick: e => {
                this.closeOverlayButtonClicked(e);
            }, class: "content-overlay__close", title: this.closeTitle }, !this.hasCloseButton && (h("svg", { key: 'ab3a9eae967ae1aa65a62c4f97bf346ff5b7ae56', xmlns: "http://www.w3.org/2000/svg", fill: "none", viewBox: "0 0 24 24", "stroke-width": "1.5", stroke: "currentColor" }, h("path", { key: '66e8c160d73fa53abb9d9d1e1b10421003fd13f9', "stroke-linecap": "round", "stroke-linejoin": "round", d: "M6 18 18 6M6 6l12 12" }))), h("slot", { key: '680eda3171fd38dd5b00963eef82c1221b94d663', name: "closeButton" })), h("slot", { key: 'd8c34c5bb0e24a35b8351c3ea364c09509211c2c', name: "header" })), h("div", { key: '10415922744fe91d30afc2d7d51f379275658e0f', class: "content-overlay__content", ref: (el) => (this.contentSlot = el) }, h("slot", { key: '6564b04487863aaaeb121ab90e1361aca5fa2adc', name: "content" }), h("slot", { key: 'dac056b616dc5468eca8603c8ad7c48489224411' })))))));
    }
    componentDidRender() {
        if (this.open === 'true') {
            this.openOverlay();
        }
        if (this.isActive && this.focusTrap && !this.focusTrap.active) {
            try {
                this.focusTrap.activate();
            }
            catch (e) {
                console.log(e);
            }
        }
    }
    static get is() { return "content-overlay"; }
    static get originalStyleUrls() {
        return {
            "$": ["content-overlay.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["content-overlay.css"]
        };
    }
    static get properties() {
        return {
            "target": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "target",
                "reflect": false
            },
            "open": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "open",
                "reflect": true,
                "defaultValue": "'false'"
            },
            "fullScreen": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveBoolean",
                    "resolved": "string",
                    "references": {
                        "ResponsiveBoolean": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveBoolean"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Boolean"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "full-screen",
                "reflect": true,
                "defaultValue": "'true'"
            },
            "classes": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": ""
                },
                "attribute": "classes",
                "reflect": false
            },
            "horizontal": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ContentLineTextAlignOptions",
                    "resolved": "\"center\" | \"default\" | \"left\" | \"right\"",
                    "references": {
                        "ContentLineTextAlignOptions": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentLineTextAlignOptions"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "default"
                        }, {
                            "name": "options",
                            "text": "left"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "right"
                        }, {
                            "name": "responsive",
                            "text": undefined
                        }],
                    "text": "A pipe delimited setting for horizontal alignment. This only applies when fullscreen mode is activated."
                },
                "attribute": "horizontal",
                "reflect": true,
                "defaultValue": "'center'"
            },
            "vertical": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ContentVerticalAlignments",
                    "resolved": "\"bottom\" | \"center\" | \"top\"",
                    "references": {
                        "ContentVerticalAlignments": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ContentVerticalAlignments"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "top"
                        }, {
                            "name": "options",
                            "text": "center"
                        }, {
                            "name": "options",
                            "text": "bottom"
                        }],
                    "text": "A pipe delimited setting for vertical alignment. This only applies when fullscreen mode is activated."
                },
                "attribute": "vertical",
                "reflect": true,
                "defaultValue": "'center'"
            },
            "closeTitle": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "String"
                        }],
                    "text": "Title attribute value for the close button."
                },
                "attribute": "close-title",
                "reflect": false,
                "defaultValue": "'Close Overlay'"
            },
            "position": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "ResponsiveString",
                    "resolved": "string",
                    "references": {
                        "ResponsiveString": {
                            "location": "import",
                            "path": "../../typings",
                            "id": "src/typings.d.ts::ResponsiveString"
                        }
                    }
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "type",
                            "text": "Select"
                        }, {
                            "name": "options",
                            "text": "absolute"
                        }, {
                            "name": "options",
                            "text": "fixed"
                        }, {
                            "name": "options",
                            "text": "static"
                        }, {
                            "name": "options",
                            "text": "sticky"
                        }],
                    "text": "Position of the overlay."
                },
                "attribute": "position",
                "reflect": true,
                "defaultValue": "'fixed'"
            }
        };
    }
    static get states() {
        return {
            "isActive": {},
            "activeClass": {},
            "classString": {},
            "contentSlot": {},
            "isFullScreen": {},
            "hasCloseButton": {},
            "originalMarkup": {}
        };
    }
    static get events() {
        return [{
                "method": "contentOverlayActivated",
                "name": "contentOverlayActivated",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "HTMLElement",
                    "resolved": "HTMLElement",
                    "references": {
                        "HTMLElement": {
                            "location": "global",
                            "id": "global::HTMLElement"
                        }
                    }
                }
            }];
    }
    static get methods() {
        return {
            "openOverlay": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            },
            "closeOverlay": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": []
                }
            }
        };
    }
    static get elementRef() { return "hostElement"; }
    static get listeners() {
        return [{
                "name": "toggle",
                "method": "onContentBlockToggle",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "open",
                "method": "onContentBlockActivated",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "close",
                "method": "onContentBlockDeactivated",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=content-overlay.js.map
